@@ -100,31 +100,29 @@ def smooth_knn_dist(distances, k, n_iter=64, local_connectivity=1.0, bandwidth=1
             index = int(np.floor(local_connectivity))
             interpolation = local_connectivity - index
             if index > 0:
-                if interpolation <= SMOOTH_K_TOLERANCE:
-                    rho[i] = non_zero_dists[index - 1]
-                else:
-                    rho[i] = non_zero_dists[index - 1] + interpolation * (
-                        non_zero_dists[index] - non_zero_dists[index - 1]
-                    )
+                rho[i] = non_zero_dists[index - 1]
+                if interpolation > SMOOTH_K_TOLERANCE:
+                    rho[i] += interpolation * (non_zero_dists[index] - non_zero_dists[index - 1])
             else:
                 rho[i] = interpolation * non_zero_dists[0]
         elif non_zero_dists.shape[0] > 0:
             rho[i] = np.max(non_zero_dists)
-        else:
-            rho[i] = 0.0
 
         for n in range(n_iter):
 
             psum = 0.0
             for j in range(1, distances.shape[1]):
-                d = max(0, (distances[i, j] - rho[i]))
-                psum += np.exp(-(d / mid))
-            val = psum
+                d = distances[i, j] - rho[i]
+                if d > 0:
+                    psum += np.exp(-(d / mid))
+                else:
+                    psum += 1.0
 
-            if np.fabs(val - target) < SMOOTH_K_TOLERANCE:
+
+            if np.fabs(psum - target) < SMOOTH_K_TOLERANCE:
                 break
 
-            if val > target:
+            if psum > target:
                 hi = mid
                 mid = (lo + hi) / 2.0
             else:
@@ -908,7 +906,7 @@ def simplicial_set_embedding(
     """
     graph = graph.tocoo()
     graph.sum_duplicates()
-    n_vertices = graph.shape[0]
+    n_vertices = graph.shape[1]
 
     if n_epochs <= 0:
         # For smaller datasets we can use more epochs
@@ -1417,18 +1415,22 @@ class UMAP(BaseEstimator):
                 self._distance_func = self.metric
             elif self.metric in dist.named_distances:
                 self._distance_func = dist.named_distances[self.metric]
+            elif self.metric == 'precomputed':
+                warn('Using precomputed metric; transform will be unavailable for new data')
             else:
                 raise ValueError(
                     "Metric is neither callable, " + "nor a recognised string"
                 )
-            self._dist_args = tuple(self._metric_kwds.values())
 
-            self._random_init, self._tree_init = make_initialisations(
-                self._distance_func, self._dist_args
-            )
-            self._search = make_initialized_nnd_search(
-                self._distance_func, self._dist_args
-            )
+            if self.metric != 'precomputed':
+                self._dist_args = tuple(self._metric_kwds.values())
+
+                self._random_init, self._tree_init = make_initialisations(
+                    self._distance_func, self._dist_args
+                )
+                self._search = make_initialized_nnd_search(
+                    self._distance_func, self._dist_args
+                )
 
         if y is not None:
             if self.target_metric == "categorical":
@@ -1456,7 +1458,6 @@ class UMAP(BaseEstimator):
                     False,
                     1.0,
                     1.0,
-                    1.0,
                     False,
                 )
                 # product = self.graph_.multiply(target_graph)
@@ -1544,6 +1545,9 @@ class UMAP(BaseEstimator):
 
         if self._sparse_data:
             raise ValueError("Transform not available for sparse input.")
+        elif self.metric == 'precomputed':
+            raise ValueError("Transform  of new data not available for "
+                             "precomputed metric.")
 
         X = check_array(X, dtype=np.float32, order="C")
         random_state = check_random_state(self.transform_seed)
@@ -1617,7 +1621,7 @@ class UMAP(BaseEstimator):
             head,
             tail,
             n_epochs,
-            X.shape[0],
+            graph.shape[1],
             epochs_per_sample,
             self._a,
             self._b,
